@@ -1,15 +1,17 @@
 import datetime
 import json
 import pytest
+import re
 import requests_mock
 import tests.test_json
 import dydx.util as utils
+import dydx.constants as consts
 from dydx.client import Client
 
 PRIVATE_KEY_1 = '0x4f3edf983ac636a65a842ce7c78d9aa706d3b113bce9c46f30d7d21715b23b1d'  # noqa: E501
 PRIVATE_KEY_2 = '0x6cbed15c793ce57650b9877cf6fa156fbef513c4e6134f022a85b1ffdd59b2a1'  # noqa: E501
-ADDRESS_1 = '0x90f8bf6a479f320ead074411a4b0e7944ea8c9c1'
-ADDRESS_2 = '0xffcf8fdee72ac11b5c542428b35eef5769c409f0'
+ADDRESS_1 = '0x90F8bf6A479f320ead074411a4B0e7944Ea8c9C1'
+ADDRESS_2 = '0xFFcf8FDEE72ac11b5c542428B35EEF5769C409f0'
 ADDRESS_1_NO_PREFIX = ADDRESS_1[2:]
 ADDRESS_2_NO_PREFIX = ADDRESS_2[2:]
 ORDER = {
@@ -28,8 +30,8 @@ ORDER_HASH = '0x998ec84efeb9d5b2e20820722d90a9430ec7085ad45bbd7b2cd6b312abe294c5
 CANCEL_ORDER_HASH = '0x54da43ec40e5ae61b2dac3d9068cd56d257459bc105ad0317857b7b4f66e101c'  # noqa: E501
 ORDER_SIGNATURE = '0x342f7533477aff89c3d25facdecb3875a68ccb5271a79dca64d19c822a6a8d560dba1ce392a50d7cd0d76ee45cfd8e6627764b012970bf43f6f8fd61677cf2ba1c01'  # noqa: E501
 CANCEL_ORDER_SIGNATURE = '0xe1381f81b47132cc23809ddff717b40d52f3a4c7cbb49f85aadf2f893c6f433c05d5cc4143694630ac504429b1c325251dec3dde9fec6c02f95246663c0f4b7f1c01'  # noqa: E501
-MAX_SOLIDITY_UINT = 115792089237316195423570985008687907853269984665640564039457584007913129639935  # noqa: E501
 PAIRS = ['WETH-DAI', 'DAI-WETH']
+LOCAL_NODE = 'http://0.0.0.0:8545'
 
 
 class TestClient():
@@ -46,15 +48,10 @@ class TestClient():
         assert client.public_address == ADDRESS_1
         assert client.account_number == 0
 
-    def test_constructor_public_address(self):
-        client = Client(PRIVATE_KEY_1, public_address=ADDRESS_1)
-        assert client.public_address == ADDRESS_1
-        assert client.account_number == 0
-
     def test_constructor_account_number(self):
-        client = Client(PRIVATE_KEY_1, account_number=MAX_SOLIDITY_UINT)
-        assert client.account_number == MAX_SOLIDITY_UINT
-        assert str(MAX_SOLIDITY_UINT) == '115792089237316195423570985008687907853269984665640564039457584007913129639935'  # noqa: E501
+        client = Client(PRIVATE_KEY_1, account_number=consts.MAX_SOLIDITY_UINT)
+        assert client.account_number == consts.MAX_SOLIDITY_UINT
+        assert str(consts.MAX_SOLIDITY_UINT) == '115792089237316195423570985008687907853269984665640564039457584007913129639935'  # noqa: E501
 
     def test_constructor_bad_private_key(self):
         with pytest.raises(TypeError):
@@ -64,10 +61,9 @@ class TestClient():
         with pytest.raises(TypeError):
             Client()
 
-    def test_constructor_bad_public_address(self):
-        with pytest.raises(ValueError) as error:
-            Client(PRIVATE_KEY_1, public_address=ADDRESS_2)
-        assert 'private_key/public_address mismatch' in str(error.value)
+    # -----------------------------------------------------------
+    # Public API
+    # -----------------------------------------------------------
 
     # ------------ get_pairs ------------
 
@@ -314,7 +310,7 @@ class TestClient():
                 str(client.TAKER_ACCOUNT_NUMBER)
             assert abs(
                 int(body['order']['expiration']) -
-                utils.epoch_in_four_weeks()) <= 1
+                utils.epoch_in_four_weeks()) <= 10
             assert body['order']['salt'].isnumeric()
             sent_order = body['order']
             expected_signature = utils.sign_order({
@@ -374,3 +370,104 @@ class TestClient():
                 hash=ORDER_HASH
             )
             assert result == json_obj
+
+    # -----------------------------------------------------------
+    # Ethereum Transactions
+    # -----------------------------------------------------------
+
+    # ------------ set_allowance ------------
+
+    def test_set_allowance_eth_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.set_allowance(market=0)
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_set_allowance_dai_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.set_allowance(market=1)
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_set_allowance_unknown_failure(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        with pytest.raises(ValueError) as error:
+            client.set_allowance(market=3)
+        assert 'Invalid market number' in str(error.value)
+
+    # ------------ enable_limit_orders ------------
+
+    def test_enable_limit_orders_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.enable_limit_orders()
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    # ------------ deposit ------------
+
+    def test_deposit_eth_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.deposit(
+            market=0,
+            wei=1000
+        )
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_deposit_dai_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.deposit(
+            market=1,
+            wei=1000
+        )
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_deposit_unknown_failure(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        with pytest.raises(ValueError) as error:
+            client.deposit(
+                market=3,
+                wei=1000
+            )
+        assert 'Invalid market number' in str(error.value)
+
+    # ------------ withdraw ------------
+
+    def test_withdraw_eth_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.withdraw(
+            market=0,
+            wei=1000
+        )
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_withdraw_dai_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.withdraw(
+            market=1,
+            wei=1000
+        )
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_withdraw_unknown_failure(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        with pytest.raises(ValueError) as error:
+            client.withdraw(
+                market=3,
+                wei=1000
+            )
+        assert 'Invalid market number' in str(error.value)
+
+    # ------------ withdraw_to_zero ------------
+
+    def test_withdraw_to_zero_eth_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.withdraw_to_zero(market=0)
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_withdraw_to_zero_dai_success(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        tx_hash = client.withdraw_to_zero(market=1)
+        assert re.compile("^0x[a-f0-9]{64}").match(tx_hash)
+
+    def test_withdraw_to_zero_unknown_failure(self):
+        client = Client(PRIVATE_KEY_1, node=LOCAL_NODE)
+        with pytest.raises(ValueError) as error:
+            client.withdraw_to_zero(market=3)
+        assert 'Invalid market number' in str(error.value)
